"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  DefaultStatefulContext: () => DefaultStatefulContext,
  MemoryStore: () => MemoryStore,
  QueryCache: () => QueryCache,
  RedisStore: () => RedisStore
});
module.exports = __toCommonJS(src_exports);

// src/stores/memory.ts
var MemoryStore = class {
  state;
  name = "memory";
  constructor(config) {
    this.state = config.persistentMap;
  }
  async get(key) {
    const value = this.state.get(key);
    if (!value) {
      return Promise.resolve(void 0);
    }
    if (value.expires <= Date.now()) {
      await this.remove(key);
    }
    return Promise.resolve(value.entry);
  }
  async set(key, entry) {
    this.state.set(key, {
      expires: entry.staleUntil,
      entry
    });
    return Promise.resolve();
  }
  async remove(keys) {
    const cacheKeys = Array.isArray(keys) ? keys : [keys];
    for (const key of cacheKeys) {
      this.state.delete(key);
    }
    return Promise.resolve();
  }
};

// src/stores/redis.ts
var RedisStore = class {
  redis;
  name = "redis";
  prefix;
  constructor(config) {
    this.redis = config.redis;
    this.prefix = config.prefix || "sbch";
  }
  buildCacheKey(key) {
    return [this.prefix, key].join("::");
  }
  async get(key) {
    const res = await this.redis.get(this.buildCacheKey(key));
    if (!res) return;
    return JSON.parse(res);
  }
  async set(key, entry) {
    await this.redis.set(
      this.buildCacheKey(key),
      JSON.stringify(entry),
      "PXAT",
      entry.staleUntil
    );
  }
  async remove(keys) {
    const cacheKeys = (Array.isArray(keys) ? keys : [keys]).map(
      (key) => this.buildCacheKey(key).toString()
    );
    this.redis.del(...cacheKeys);
  }
};

// src/context.ts
var DefaultStatefulContext = class {
  waitUntil(_p) {
  }
};

// src/key.ts
var import_postgrest_core = require("@supabase-cache-helpers/postgrest-core");
var SEPARATOR = "$";
function encode(query) {
  if (!(0, import_postgrest_core.isPostgrestBuilder)(query)) {
    throw new Error("Query is not a PostgrestBuilder");
  }
  const parser = new import_postgrest_core.PostgrestParser(query);
  return [
    parser.schema,
    parser.table,
    parser.queryKey,
    parser.bodyKey ?? "null",
    `count=${parser.count}`,
    `head=${parser.isHead}`,
    parser.orderByKey
  ].join(SEPARATOR);
}

// src/swr-cache.ts
var SwrCache = class {
  ctx;
  store;
  fresh;
  stale;
  constructor({ ctx, store, fresh, stale }) {
    this.ctx = ctx;
    this.store = store;
    this.fresh = fresh;
    this.stale = stale;
  }
  /**
   * Return the cached value
   *
   * The response will be `undefined` for cache misses or `null` when the key was not found in the origin
   */
  async get(key) {
    const res = await this._get(key);
    return res.value;
  }
  async _get(key) {
    const res = await this.store.get(key);
    const now = Date.now();
    if (!res) {
      return { value: void 0 };
    }
    if (now >= res.staleUntil) {
      this.ctx.waitUntil(this.remove(key));
      return { value: void 0 };
    }
    if (now >= res.freshUntil) {
      return { value: res.value, revalidate: true };
    }
    return { value: res.value };
  }
  /**
   * Set the value
   */
  async set(key, value, opts) {
    const now = Date.now();
    return this.store.set(key, {
      value,
      freshUntil: now + (opts?.fresh ?? this.fresh),
      staleUntil: now + (opts?.stale ?? this.stale)
    });
  }
  /**
   * Removes the key from the cache.
   */
  async remove(key) {
    return this.store.remove(key);
  }
  async swr(key, loadFromOrigin, opts) {
    const res = await this._get(key);
    const { value, revalidate } = res;
    if (typeof value !== "undefined") {
      if (revalidate) {
        this.ctx.waitUntil(
          loadFromOrigin(key).then((res2) => this.set(key, res2, opts))
        );
      }
      return value;
    }
    const loadedValue = await loadFromOrigin(key);
    this.ctx.waitUntil(this.set(key, loadedValue));
    return loadedValue;
  }
};

// src/tiered-store.ts
var TieredStore = class {
  ctx;
  tiers;
  name = "tiered";
  /**
   * Create a new tiered store
   * Stored are checked in the order they are provided
   * The first store to return a value will be used to populate all previous stores
   *
   *
   * `stores` can accept `undefined` as members to allow you to construct the tiers dynamically
   * @example
   * ```ts
   * new TieredStore(ctx, [
   *   new MemoryStore(..),
   *   process.env.ENABLE_X_STORE ? new XStore(..) : undefined
   * ])
   * ```
   */
  constructor(ctx, stores) {
    this.ctx = ctx;
    this.tiers = stores.filter(Boolean);
  }
  /**
   * Return the cached value
   *
   * The response will be `undefined` for cache misses or `null` when the key was not found in the origin
   */
  async get(key) {
    if (this.tiers.length === 0) {
      return;
    }
    for (let i = 0; i < this.tiers.length; i++) {
      const res = await this.tiers[i].get(key);
      if (!res) {
        return;
      }
      this.ctx.waitUntil(
        Promise.all(
          this.tiers.filter((_, j) => j < i).map((t) => () => t.set(key, res))
        )
      );
      return res;
    }
  }
  /**
   * Sets the value for the given key.
   */
  async set(key, value) {
    await Promise.all(this.tiers.map((t) => t.set(key, value)));
  }
  /**
   * Removes the key from the cache.
   */
  async remove(key) {
    await Promise.all(this.tiers.map((t) => t.remove(key)));
  }
};

// src/query-cache.ts
var QueryCache = class {
  inner;
  /**
   * To prevent concurrent requests of the same data, all queries are deduplicated using
   * this map.
   */
  runningQueries = /* @__PURE__ */ new Map();
  constructor(ctx, opts) {
    const tieredStore = new TieredStore(ctx, opts.stores);
    this.inner = new SwrCache({
      ctx,
      store: tieredStore,
      fresh: opts.fresh,
      stale: opts.stale
    });
  }
  async query(query, opts) {
    const key = encode(query);
    const value = await this.inner.get(key);
    if (value) return value;
    const result = await this.dedupeQuery(query);
    if (!opts?.store || opts.store(result)) {
      await this.inner.set(key, result, opts);
    }
    return result;
  }
  async swr(query, opts) {
    return await this.inner.swr(
      encode(query),
      () => this.dedupeQuery(query),
      opts
    );
  }
  /**
   * Deduplicating the origin load helps when the same value is requested many times at once and is
   * not yet in the cache. If we don't deduplicate, we'd create a lot of unnecessary load on the db.
   */
  async dedupeQuery(query) {
    const key = encode(query);
    try {
      const querying = this.runningQueries.get(key);
      if (querying) {
        return querying;
      }
      this.runningQueries.set(key, query);
      return await query;
    } finally {
      this.runningQueries.delete(key);
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  DefaultStatefulContext,
  MemoryStore,
  QueryCache,
  RedisStore
});
//# sourceMappingURL=index.cjs.map